<?php
/**
 * This code handles AJAX POST-ed data from the personal (user) settings GUI:
 * updates the stored IMAP credentials (hostname, username, password) of the
 * ownCloud user actually logged in, for all available Roundcube instances.
 *
 * @author      Zoltán Kovács <kzoli@kzoli.hu>
 * @license     GPL-3.0 http://www.gnu.org/licenses/gpl-3.0.html
 *
 * Based on the 'external' core app by Frank Karlitschek and on
 * the 3rd party 'roundcube' app by Martin Reinhardt and David Jaedke.
 *
 * @todo        Regexp check/validation of POST-ed data.
 */

OCP\JSON::checkAppEnabled('roundcube_external');
OCP\User::checkLoggedIn();
OCP\JSON::callCheck();

// Get localization.
$l=OC_L10N::get('roundcube_external');

/**
 * Parses the POST into a sanitized, two dimensions "modifier" array
 * (this is our data structure for Roundcube credentials);
 * rows are the Roundcube instances, columns are the credentials for this instance:
 * [mail_host]      Roundcube hostname (unique)
 *   [mail_user]     IMAP username at this host
 *   [mail_password] IMAP password (plain text)
 * Omit the empty rows, except when the credentials were just deleted.
 *
 * @todo	Regexp check/validation.
 */

$modifiers = array();
\OCP\Util::writeLog('roundcube_external', 'POST data for Roundcube credentials of ' . \OCP\User::getUser() . ': ' . print_r($_POST,true), \OCP\Util::DEBUG);
for ($i = 0; $i < sizeof($_POST['mail_user']); $i++) { // enumerating credential lines sent
    if ((!empty($_POST['trashed']) && $_POST['mail_host'][$i] === $_POST['trashed']) ||
        (!empty($_POST['mail_user'][$i]) || !empty($_POST['mail_password'][$i])))
    { // not empty or just trashed lines are going to our data structure
        $modifiers[$_POST['mail_host'][$i]] = array('mail_user' => strip_tags($_POST['mail_user'][$i]),
	                                            'mail_password' => strip_tags($_POST['mail_password'][$i]));
    }
} // modifiers done
\OCP\Util::writeLog('roundcube_external', 'Roundcube credential modifiers for ' . \OCP\User::getUser() . ': ' . print_r($modifiers,true), \OCP\Util::DEBUG);

if ($modifiers) {
    // Merging the currently stored credentials with the POSTed modifiers.
    $credentials = \OCA\Roundcube_External\Roundcube_External::getCredentials();
    foreach (array_keys($modifiers) as $mail_host) { // enumerating the modifiers
	if (!in_array($mail_host, array_keys($credentials))) { // this is a new credential request
	    if (!empty($modifiers[$mail_host]['mail_user']) ||
	        !empty($modifiers[$mail_host]['mail_password'])) { // contains data
		$credentials[$mail_host] = array('mail_user' => $modifiers[$mail_host]['mail_user'],
	                                         'mail_password' => $modifiers[$mail_host]['mail_password'],);
	    }
	}
	else if (empty($modifiers[$mail_host]['mail_user']) &&
	         empty($modifiers[$mail_host]['mail_password'])) { // this is a delete request
	    unset($credentials[$mail_host]);
	}
	else { // this is a change request
	    $credentials[$mail_host] = array('mail_user' => $modifiers[$mail_host]['mail_user'],
					     // an empty password in modifier indicates no change
	                            	     'mail_password' => (!empty($modifiers[$mail_host]['mail_password']))
	                                                         ? $modifiers[$mail_host]['mail_password']
	                                                         : $credentials[$mail_host]['mail_password']);
	}
    } // merge done
    \OCP\Util::writeLog('roundcube_external', 'Roundcube credentials stored for ' . \OCP\User::getUser() . ': ' . print_r($credentials,true), \OCP\Util::DEBUG);

    // Saves the modified credentials into the user's preferences.
    if (\OCA\Roundcube_External\Roundcube_External::saveCredentials($credentials)) { // success
	\OCP\JSON::success(array('data' => array('message' => $l->t('Roundcube credentials saved.')))); }
    else { // failure
	\OCP\JSON::error(array('data' => array('message' => $l->t('Failed to save Roundcube credentials.')))); }
}
else { // Terminates with a neutral message.
    \OCP\JSON::success(array('data' => array('message' => $l->t('No change happened.'))));
}