<?php
/**
 * This code handles Ajax POST-ed data from the administrative settings GUI:
 * updates the list (name, URL and icon) of available Roundcube instances.
 * Minimally checks data and overwrites the existing configuration with them.
 *
 * @author      Zoltán Kovács <kzoli@kzoli.hu>
 * @license     GPL-3.0 http://www.gnu.org/licenses/gpl-3.0.html
 *
 * Based on the 'external' core app by Frank Karlitschek and on
 * the 3rd party 'roundcube' app by Martin Reinhardt and David Jaedke.
 *
 * @todo	Regexp check/validation of POST-ed data.
 */

OCP\JSON::checkAppEnabled('roundcube_external');
OCP\User::checkAdminUser();
OCP\JSON::callCheck();

// Get internationalization.
$l=OC_L10N::get('roundcube_external');

/**
 * Parses the POST into a sanitized, two dimensions array
 * (this is our data structure for Roundcube instances);
 * rows are the Roundcube instances, columns are the parameters for this instance:
 * [0..n]	     actual Roundcube instance number (numeric key)
 *   [site_name]     arbitrary name, identifying this instance for users
 *   [site_url]	     instance's URL [http(s)://fully.qualified.hostname/...]
 *   [site_icon]     icon for this instance in the Applications menu.
 * Omit the empty rows POSTed.
 *
 * @todo        Regexp check/validation.
 */

$sites = array();
\OCP\Util::writeLog('roundcube_external', 'POST data for Roundcube instances: ' . print_r($_POST,true), \OCP\Util::DEBUG);
for ($i = 0; $i < sizeof($_POST['site_name']); $i++) { // enumerating instance lines sent
    if (!empty($_POST['site_name'][$i]) && !empty($_POST['site_url'][$i]))
    { // not empty lines are going to our data structure
	array_push($sites, array(strip_tags($_POST['site_name'][$i]),
	                         strip_tags($_POST['site_url'][$i]),
	                         strip_tags($_POST['site_icon'][$i]) ));
    }
} // sites done
\OCP\Util::writeLog('roundcube_external', 'Roundcube instances: ' . print_r($sites,true), \OCP\Util::DEBUG);

if ($sites) {
    // Formally checks the site URLs (TODO!).
    foreach($sites as $site) {
	if (strpos($site[1], 'http://') === 0) continue;
	if (strpos($site[1], 'https://') === 0) continue;
//  	if (strncmp($site[1], '/', 1) === 0) continue;
	// On error terminates with message.
	OC_JSON::error(array('data' => array('message' => $l->t('Please enter valid urls - they have to start with either http:// or https://') )));
	return;
    } // check done
    \OCP\Util::writeLog('roundcube_external', 'Roundcube instances stored: ' . print_r($sites,true), \OCP\Util::DEBUG);

    // Replace instances into the app's values.
    if (\OCA\Roundcube_External\Roundcube_External::saveSites($sites)) { // success
        \OCP\JSON::success(array('data' => array('message' => $l->t('Roundcube sites saved.')))); }
    else { // failure
        \OCP\JSON::error(array('data' => array('message' => $l->t('Failed to save Roundcube sites.')))); }
}
else { // Terminates with a neutral message.
    \OCP\JSON::success(array('data' => array('message' => $l->t('No change happened.'))));
}
